#!/usr/bin/perl
#
# This file is part of BG-Billing system.
# Any use outside BG-Billing infrastructure is forbidden.
# Copyright 2007 Bitel Ltd. (www.bitel.ru)
# All right reserved.
#

use IO::File;
use Fcntl qw(:flock);
use POSIX;
use Switch;

require "conf";

my $uid = 1;
my $gid = 1;

my $name = $ARGV[0];

chomp( $name );

my $logFile = $sourcelog_path.$name;
#my $file = new IO::File ":bytes", "< $logFile"
open( FILE, "<:bytes:raw", $logFile ) or die "Cannot open $logFile : $!";
flock( FILE, LOCK_SH );

my $logYY = undef;
my $logMM = undef;
my $logDD = undef;

if ( $name =~ /AMA\.(\d\d)(\d\d)(\d\d)\.ama/ )
{
    $logDD = $3;
    $logMM = $2;
    $logYY = "20".$1;
			    
}
elsif ( $name =~ /.*(\d\d)(\d\d)(\d\d)\.ama/ )
{
	$logDD = $1;
	$logMM = $2;
	$logYY = "20".$3;
}

print "Processing log $logFile.";
#if ( $logYY != undef ) {
#	print " Date: $logDD.$logMM.$logYY.";
#}
print "\n";

my $shortNumbers = ();


#  100   DATE / TIME / DURATION       fixed       none : 11          x
#  101   PARTNER DIRECTORY NUMBER     variable    calculated         x
#  102   SERVICE INFO                 fixed       none : 4          [x]
#  103   CHARGE UNITS FOR CONNECTION  fixed       none : 4          opt
#  105   TRUNK ID INCOMING            fixed       none : 9          [x]
#  106   TRUNK ID OUTGOING            fixed       none : 9          [x]
#  107   TRUNK ID INCOMING WITH CIC   fixed       none : 10        [opt]
#  108   TRUNK ID OUTGOING WITH CIC   fixed       none : 10        [opt]
#  110   CONNECTION IDENTIFICATION    fixed       on byte 2        [opt]
#  117   PROJECT SPECIFIC DATA        variable    on byte 2          *
#  119   TRANSMISSION MEDIUM REQUIRED fixed       none : 2          [x]
#  120   CATEGORY                     fixed       none : 2          [x]
#  128   NEW DESTINATION              variable    calculated        [x]
#  130   TRAFFIC QUALITY DATA         fixed       on byte 2         [x]
#  134   DURATIONS BEFORE ANSWER      fixed       on byte 2         [x]
#  138   B METER PULSES               fixed       on byte 2         opt
#  142   CALLING PARTY NUMBER         variable    on byte 2         opt
#  145   CAC NUMBER                   variable    on byte 2        [opt]
#  146   THIRD PARTY NUMBER           variable    on byte 2        [opt]
#  147   TARIFF INFO                  variable    on byte 2        [opt]
#  148   CAC NEW DESTINATION          variable    on byte 2        [opt]
#  149   GLOBAL CHARGE REFERENCE      variable    on byte 2         opt
#  150   AUDIT TRAIL                  fixed       on byte 2         opt
#  154   ADDITIONAL CALL INDICATORS   variable    on byte 2         opt
#  157   OUTGOING PA SLAVE            variable    on byte 2         opt
#        IDENTIFICATION


my $data;

my $count = 0;

my $lastFilePos;
my $filePos;

my $DATA = ();


while ( read( FILE, $data, 1 ) ) {# && $count < 100

	$lastFilePos = $filePos;
	$filePos = tell( FILE );

	@f = unpack ( 'C', $data );

	$count++;

	if ( $f[0] == 132 )
	{
		my $recordId = $f[0];

		my @packages = ();

		read( FILE, $data, 7 );
		@f = unpack ( 'SCCCCC', $data );
	
		my $recordLength = $f[0];
	
		my $iacama = ($f[1]&0x2)>0?1:0;
		my $connection = ($f[1]&0x8)>0?1:0;
		my $analog = ($f[2]&0x8)>0?1:0;
		my $isdn = ($f[2]&0x16)>0?1:0;

		my $noAnswer = ($f[3]&0x4)>0?1:0;
		my $sequenceNumber = ($f[3]&0x40)>0?1:0;
		my $checksum = ($f[3]&0x80)>0?1:0;
	
		my $recordSeq = ($f[4]&0xF0)>>4;
		my $chargeStatus = ($f[4])&0xF;
	
		my $lacLength = ($f[5]>>5)&0x7;
		my $ownerIdLength = ($f[5])&0x1F;

		my $len = ceil($ownerIdLength/2);
		read( FILE, $data, $len ) == $len or die "Failed to read next block of record!";

		@num = unpack ( 'H['.$ownerIdLength.']', $data );
		my $owner = substr( $num[0], 0, $ownerIdLength );

		$len = $recordLength - 8 - $len;

#		if ( !$iacama )
#		{
#			print "Not IACAMA flag, record will be dropped. $count $iacama\n";
#			next;
#		}
		if ( !$connection )
		{
#			print "Not CONNECTION flag, record will be dropped. $count $iacama $recordSeq $chargeStatus o:$owner\n";
			seek( FILE, $filePos+$recordLength-1, 0 );
			next;
		}

		my %beginDateTime = ( 'undef', 1 );
		my %endDateTime = ( 'undef', 1 );

		my $partnerNumber = undef;
		my $newDestination = undef;

		my $callingPartyNumber;
		my $callingPartyNumberNADI;
		my $callingPartyNumberNPI;

		my %serviceInfo;

		my $chargeUnits;

		my %trafficQuality;

		my $connectionIdentification;

		my $trunkIdIncoming = 0;
		my $trunkIdOutgoing = 0;

		my $ppp = 0;

		my $tmp = '';
	
		while ( $len > 0 )
		{
			read( FILE, $data, 1 );
			@f = unpack ( 'C', $data );

			$packages[++$#packages] = $f[0];

			#101, 105, 106, 128, 145, 146, 103, 110
			$tmp .= $f[0].' ';

			if ( $f[0] == 100 )#  100   DATE / TIME / DURATION       fixed       none : 11          x
			{
				$ppp = 1;

				read( FILE, $data, 10 ) == 10 or die "Failed to read next block of record!";
				@f = unpack ( 'CCCCCCCCS', $data );

				my $begin = ($f[6]&0x40)==0?1:0;
	
				my $time;
				if ( $begin )
				{
					$time = \%beginDateTime;
				}
				else
				{
					$time = \%endDateTime;
				}

					my $month = $f[1];
					$month =~ s/^(\d)$/0$1/;
					my $dd = $f[2];
					$dd =~ s/^(\d)$/0$1/;
					my $hh = $f[3];
					$hh =~ s/^(\d)$/0$1/;
					my $mm = $f[4];
					$mm =~ s/^(\d)$/0$1/;
					my $ss = $f[5];
					$ss =~ s/^(\d)$/0$1/;

					$time->{ 'undef' } = 0;
					$time->{ 'year' } = $f[0] + 2000;
					$time->{ 'month' } = $month;
					$time->{ 'day' } = $dd;
					$time->{ 'hour' } = $hh;
					$time->{ 'minute' } = $mm;
					$time->{ 'second' } = $ss;

					$time->{ 'beginDateTime' } = $begin;
					$time->{ 'timeSecure' } = ($f[6]&0x80)>0?1:0;
					$time->{ 'duration' } = ($f[7]&0xFF) | (($f[8]&0xFFFF)<<8); #<<8)

					if ( $logYY == undef ) {
						$logDD = $time->{ 'day' };
						$logMM = $time->{ 'month' };
						$logYY = $time->{ 'year' };
						print " Date: $logDD.$logMM.$logYY.";
					}
					
				$len -= 11;
			}
			elsif ( $f[0] == 101 )#  101   PARTNER DIRECTORY NUMBER     variable    calculated         x
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
	
				my $numLen = ($f[0]&0x1F);
				my $l = ceil($numLen/2);
	
				read( FILE, $data, $l ) == $l or die "Failed to read next block of record!";
				@num = unpack ( 'H['.$numLen.']', $data );
				$partnerNumber = substr( $num[0], 0, $numLen );
	
				$len -= 2 + $l;
			}
			elsif ( $f[0] == 102 )#  102   SERVICE INFO                 fixed       none : 4          [x]
			{
				read( FILE, $data, 3 ) == 3 or die "Failed to read next block of record!";
				@f = unpack ( 'CCC', $data );
	
				$serviceInfo{ 'indicator' } = $f[0];
				$serviceInfo{ 'additional' } = $f[1];
				$serviceInfo{ 'blueBook' } = ($f[2]&0x1)>0?1:0;
	
				$len -= 4;
			}
			elsif ( $f[0] == 103 )#  103   CHARGE UNITS FOR CONNECTION  fixed       none : 4          opt
			{
				read( FILE, $data, 3 ) == 3 or die "Failed to read next block of record!";
				@f = unpack ( 'CS', $data );
				$chargeUnits = (($f[0]&0xFF)<<16) | ($f[1]&0xFFFF);   #<<16)
	
				$len -= 4;
			}
			elsif ( $f[0] == 105 )#  105   TRUNK ID INCOMING            fixed       none : 9          [x]
			{
				read( FILE, $data, 8 ) == 8 or die "Failed to read next block of record!";
				@f = unpack ( 'A[6]S', $data );

				$trunckIdIncoming = $f[1];

				$len -= 9;
			}
			elsif ( $f[0] == 106 )#  106   TRUNK ID OUTGOING            fixed       none : 9          [x]
			{
				read( FILE, $data, 8 ) == 8 or die "Failed to read next block of record!";
				@f = unpack ( 'A[6]S', $data );

				$trunckIdOutgoing = $f[1];
#print "out";
				$len -= 9;
			}
			elsif ( $f[0] == 107 )#  107   TRUNK ID INCOMING WITH CIC   fixed       none : 10        [opt]
			{
				read( FILE, $data, 9 ) == 9 or die "Failed to read next block of record!";
				@f = unpack ( 'A[6]SC', $data );

				$trunkIdIncoming = $f[2];

				$len -= 10;
			}
			elsif ( $f[0] == 108 )#  108   TRUNK ID OUTGOING WITH CIC   fixed       none : 10        [opt]
			{
				read( FILE, $data, 9 ) == 9 or die "Failed to read next block of record!";
				@f = unpack ( 'A[6]SC', $data );

				$trunkIdOutgoing = $f[2];

				$len -= 10;
			}
			elsif ( $f[0] == 110 )#  110   CONNECTION IDENTIFICATION    fixed       on byte 2        [opt]
			{
				read( FILE, $data, 5 ) == 5 or die "Failed to read next block of record!";
				@f = unpack ( 'CL', $data );
	
				$connectionIdentification = $f[1];

				$len -= 6;
			}
			elsif ( $f[0] == 117 )#  117   PROJECT SPECIFIC DATA        variable    on byte 2          *
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );

				$len -= $f[0];
			}
			elsif ( $f[0] == 119 )#  119   TRANSMISSION MEDIUM REQUIRED fixed       none : 2          [x]
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";

				$len -= 2;
			}
			elsif ( $f[0] == 120 )#  120   CATEGORY                     fixed       none : 2          [x]
			#Category
			#	Value Meaning
			#	0     undefined
			#	1     subscriber
			#	2     priority subscriber
			#	3     test equipment
			#	4     coin box
			#	5     operator national
			#	6     data transmission national
			#	7     subscriber or operator international w/o forward transfer
			#	8     data transmission international
			#	9     priority subscriber international
			#	10    operator international with forward transfer
			#	11-15 reserved for project specific applications
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				#@f = unpack ( 'C', $data );
	
				$len -= 2;
			}
			elsif ( $f[0] == 128 )#  128   NEW DESTINATION              variable    calculated        [x]
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
	
				my $numLen = ($f[0]&0x1F);
				my $l = ceil($numLen/2);
	
				read( FILE, $data, $l ) == $l or die "Failed to read next block of record!";
				@num = unpack ( 'H*', $data );
				$newDestination = substr( $num[0], 0, $numLen );

				$len -= 2 + $l;
			}
			elsif ( $f[0] == 130 )#  130   TRAFFIC QUALITY DATA         fixed       on byte 2         [x]
			{
				read( FILE, $data, 4 ) == 4 or die "Failed to read next block of record!";
				@f = unpack ( 'CSC', $data );
	
				$trafficQuality{ 'cause' } = $f[1];
				$trafficQuality{ 'coding' } = ($f[2]&0x60)>>5;
				$trafficQuality{ 'location' } = ($f[2]&0xF);
	
				$len -= 5;
			}
			elsif ( $f[0] == 134 )#  134   DURATIONS BEFORE ANSWER      fixed       on byte 2         [x]
			{
				read( FILE, $data, 5 ) == 5 or die "Failed to read next block of record!";
				#@f = unpack ( 'CSS', $data );
	
				$len -= 6;
			}
			elsif ( $f[0] == 138 )#  138   B METER PULSES               fixed       on byte 2         opt
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );

				$len -= $f[0];
			}
			elsif ( $f[0] == 142 )#  142   CALLING PARTY NUMBER         variable    on byte 2         opt
			{
				read( FILE, $data, 4 ) == 4 or die "Failed to read next block of record!";
				@f = unpack ( 'CCCC', $data );

				$callingPartyNumberNADI = @f[1]&0x7F;
				$callingPartyNumberNPI = @f[2]&0x70;
				my $numDigits = @f[3]&0x1F;

				$l = @f[0]-5;
				read( FILE, $data, $l ) == $l or die "Failed to read next block of record!";

				@f = unpack ( "H[$numDigits]", $data );
				$callingPartyNumber = $f[0];

				$len -= $l+5;
			}
			elsif ( $f[0] == 145 )#  145   CAC NUMBER                   variable    on byte 2        [opt]
			{
				#print "CAC\n";
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				#seek( FILE, $f[0]-2, SEEK_CUR );
				read( FILE, $data, $f[0] ) == $f[0] or die "Failed to read next block of record!";
				@f = unpack ( 'CH*', $data );

				#print 'CAC: '.($f[0]>>5)." ".(($f[0]>>4)&&0x3)." ".($f[0]&&0x7);

				$len -= $f[0];
			}
			elsif ( $f[0] == 146 )#  146   THIRD PARTY NUMBER           variable    on byte 2        [opt]
			#Third party number
			#In case of call diversion (redirection). Package 146 is present in the record for the redirecting party and
			#contains the DN of the original calling party (resp. the DN of the previous party in case of MFC-
			#signalling). The flags F1 till F6 are not relevant.
			{
				read( FILE, $data, 4 ) == 4 or die "Failed to read next block of record!";
				@f = unpack ( 'CCCC', $data );

				my $numLen = ($f[3]);
				my $l = ceil($numLen/2);

				read( FILE, $data, $l ) == $l or die "Failed to read next block of record!";
				@num = unpack ( 'H*', $data );

				#$partnerNumber = substr( $num[0], 0, $numLen );

				$len -= $l+5;
			}
			elsif ( $f[0] == 147 )#  147   TARIFF INFO                  variable    on byte 2        [opt]
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );

				$len -= $f[0];
			}
			elsif ( $f[0] == 148 )#  148   CAC NEW DESTINATION          variable    on byte 2        [opt]
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );
#print "CAC new destination\n";
				$len -= $f[0];
			}
			elsif ( $f[0] == 149 )#  149   GLOBAL CHARGE REFERENCE      variable    on byte 2         opt
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );

				$len -= $f[0];
			}
			elsif ( $f[0] == 150 )#  150   AUDIT TRAIL                  fixed       on byte 2         opt
			{
				read( FILE, $data, 7 ) == 7 or die "Failed to read next block of record!";
				#@f = unpack ( 'CSS', $data );

				$len -= 8;
			}
			elsif ( $f[0] == 154 )#  154   ADDITIONAL CALL INDICATORS   variable    on byte 2         opt
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );

				$len -= $f[0];
			}
			elsif ( $f[0] == 154 )#  157   OUTGOING PA SLAVE IDENTIFICATION            variable    on byte 2         opt
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );

				$len -= $f[0];
			}
			elsif ( $f[0] == 112 )#unknown, variable length, on second byte
			{
				read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
				@f = unpack ( 'C', $data );
				seek( FILE, $f[0]-2, SEEK_CUR );
	
				$len -= $f[0];
			}
=pod			elsif ( $f[0] == 116 )#unknown
			{
				read( FILE, $data, 11 ) == 11 or die "Failed to read next block of record!";
				@f = unpack ( 'CCCCCCCCS', $data );

				my $begin = ($f[6]&0x40)==0?1:0;
	
				my $time;
				if ( $begin )
				{
					$time = \%beginDateTime;
				}
				else
				{
					$time = \%endDateTime;
				}

					my $month = $f[1];
					$month =~ s/^(\d)$/0$1/;
					my $dd = $f[2];
					$dd =~ s/^(\d)$/0$1/;
					my $hh = $f[3];
					$hh =~ s/^(\d)$/0$1/;
					my $mm = $f[4];
					$mm =~ s/^(\d)$/0$1/;
					my $ss = $f[5];
					$ss =~ s/^(\d)$/0$1/;

					$time->{ 'undef' } = 0;
					$time->{ 'year' } = $f[0] + 2000;
					$time->{ 'month' } = $month;
					$time->{ 'day' } = $dd;
					$time->{ 'hour' } = $hh;
					$time->{ 'minute' } = $mm;
					$time->{ 'second' } = $ss;

					$time->{ 'beginDateTime' } = $begin;
					$time->{ 'timeSecure' } = ($f[6]&0x80)>0?1:0;
					$time->{ 'duration' } = ($f[7]&0xFF) | (($f[8]&0xFFFF)<<8); #<<8)

					if ( $logYY == undef ) {
						$logDD = $time->{ 'day' };
						$logMM = $time->{ 'month' };
						$logYY = $time->{ 'year' };
						print " Date: $logDD.$logMM.$logYY.";
					}
					
				$len -= 12;
			}
			elsif ( $f[0] == 113 )#unknown, fixed length
			{
				seek( FILE, 11, SEEK_CUR );
	
				$len -= 12;
			}
			elsif ( $f[0] == 137 )#unknown
			{
				seek( FILE, 5, SEEK_CUR );
				$len -= 6;
			}
			elsif ( $f[0] == 16 )#unknown
			{
				seek( FILE, 2, SEEK_CUR );
				$len -= 3;
			}
#			elsif ( $f[0] == 116 )#unknown
#			{
#				#read( FILE, $data, 1 ) == 1 or die "Failed to read next block of record!";
#				#@f = unpack ( 'C', $data );
#				#seek( FILE, $f[0]-2, SEEK_CUR );
#				seek( FILE, -13-1, SEEK_CUR );
#				read( FILE, $data, $recordLength );
#				@d = unpack('H['.($recordLength*2).']', $data );
#				print("\n$d[0]\n");
#	
#				$len -= $f[0];
#
#				die "\n$d[0]\n";
#			}
=cut
			else
			{
				seek( FILE, $filePos-1, 0 );
				read( FILE, $data, $recordLength );
				@d = unpack("H*", $data );

				print "Error! Unknown packet type $f[0], record will be dropped. FilePos: ".tell( FILE )." Record: $d[0]\n";

				$len = -1;
				seek( FILE, $filePos+$recordLength-1, 0 );
				#continue;
			}
		}

		if ( $len!=0 )
		{
			print ("Process record error\n");
		}

		#only single or final records are processing
		if ( $recordSeq!=1 && $recordSeq!=5 )
		{
			next;
		}

		if ( $checksum )
		{
			seek( FILE, $filePos-1, 0 );
			read( FILE, $data, $recordLength );
			@f = unpack("C*", $data );

			my $ch1 = 0;
			my $ch2 = 0;

			foreach my $c ( @f )
			{
				$ch1 = ( $ch1 + $c ) % 256;
				$ch2 = ( $ch1 + $ch2 ) % 256;
			}

			if ( $ch1==0 && $ch2==0 && $len!=0 )
			{
				print "Find unknown packet but checksum is valid!\n";
			}

			if ( $ch1!=0 || $ch2!=0 )
			{
				print "Checksum error! Record will be dropped. Pos: $filePos; Record: ".unpack("H*", $data );
				next;
			}
		}

		if ( ( $beginDateTime{'undef'} && $endDateTime{'undef'} ) )# || $trunkIdIncoming == undef || $trunkIdOutgoing == undef  )
		{
			print "Params error! Record will be dropped. |$trunkIdIncoming|$trunkIdOutgoing| $count. Pos: $filePos; Record: ".unpack('H*', $data )."\n";
			next;
		}
#print $owner;
		if ( !$beginDateTime{'undef'} && !$endDateTime{'undef'} )
		{
			print "Both defined $beginDateTime{'year'} ::$endDateTime{'undef'}\n";
		}

		if ( $noAnswer )
		{
			if ($callingPartyNumber != undef )
			{
				$owner = $callingPartyNumber;
			}

			if ($owner =~ /383275/ && $beginDateTime{'duration'}>0 )
			{
				print "no answ: $dd.$mm.$yy $hh:$beginDateTime{'minute'}:$beginDateTime{'second'}\t$beginDateTime{'duration'}\t$owner\t".numA2e164($owner, $lacLength)."\t$partnerNumber\t".numB2e164($partnerNumber)."\t$trunkIdIncoming\t$trunkIdOutgoing\t0\n"
			}
			#print "No answer. Dropping\n";
			next;
		}


		#01.08.2006 10:55:18 57733 3517913292 73517913292 2479292 73512479292 3 62 5
		#01.08.2006 14:08:33 75980 3517744152 73517744152 2479292 73512479292 4 34 5

#if ($recordSeq==2){
#if ($owner == "3832750006"){#3832750075
		my $yy = $beginDateTime{'year'};
		my $mm = $beginDateTime{'month'};
		my $dd = $beginDateTime{'day'};
		my $hh = $beginDateTime{'hour'};

		#   ,   0  
		if ( $yy != $logYY || $mm != $logMM || $dd != $logDD )
		{
			$yy = $logYY;
			$mm = $logMM;
			$dd = $logDD;
			$hh = '00';
		}

		if ( $yy == '2007' && $mm == '03' && $dd == '25' && $hh == '02' )
		{
			$hh = '03';
		}

		#TODO:
		my $be164 = numB2e164($partnerNumber);
#		if ( $be164 == undef)
#		{
			#print "Warn: short number found. Skipping record:\n";
			#print "$dd.$mm.$yy $hh:$beginDateTime{'minute'}:$beginDateTime{'second'}\t$beginDateTime{'duration'}\t$owner\t".numA2e164($owner)."\t$partnerNumber\t".numB2e164($partnerNumber)."\t$trunkIdIncoming\t$trunkIdOutgoing\t0\n";
#			next;
#		}
		
		if ( $logYY != undef && ( $yy!=$logYY || $mm!=$logMM || $dd!=$logDD  ) )#TODO: 
		{
			print "Warn: Out of Date! Skipping record:\n";
			print "$beginDateTime{'day'}.$beginDateTime{'month'}.$beginDateTime{'year'} $beginDateTime{'hour'}:$beginDateTime{'minute'}:$beginDateTime{'second'}\t$beginDateTime{'duration'}\t$owner\t".numA2e164($owner, $lacLength)."\t$partnerNumber\t".numB2e164($partnerNumber)."\t$trunkIdIncoming\t$trunkIdOutgoing\t0\n";
			next;
		}
#		print "$owner $lacLength $ownerIdLength\n";

		if ($callingPartyNumber != undef )
		{
			$owner = $callingPartyNumber;
		}

		if ( length($owner)<=0 ) {# 
			next;
			print "Warn! Empty num: $owner at $count, duration $beginDateTime{'duration'}\n";
#			print "Packages: $tmp";
#			print "C: $callingPartyNumber\n";
#			print "B: $partnerNumber\n";
#			seek( FILE, $filePos-1, 0 );
#			read( FILE, $data, $recordLength );
#			@d = unpack("H*", $data );
#			print "FilePos: ".tell( FILE )." Record: $d[0]\n";
		}

#		if ($owner =~ /2753005/ )
#		{
#			print "$dd.$mm.$yy $hh:$beginDateTime{'minute'}:$beginDateTime{'second'}\t$beginDateTime{'duration'}\t$owner\t".numA2e164($owner, $lacLength)."\t$partnerNumber\t".numB2e164($partnerNumber)."\t$trunkIdIncoming\t$trunkIdOutgoing\t0\n"
#		}

		my $key = $yy."_".$mm."_".$dd."_".$hh;
		my $value = "$beginDateTime{'day'}.$beginDateTime{'month'}.$beginDateTime{'year'} $beginDateTime{'hour'}:$beginDateTime{'minute'}:$beginDateTime{'second'}\t$beginDateTime{'duration'}\t$owner\t".numA2e164($owner, $lacLength)."\t$partnerNumber\t".numB2e164($partnerNumber)."\t$trunkIdIncoming\t$trunkIdOutgoing\t0\n";

		$DATA{$key} .= $value;
#}
	}
	elsif ( $f[0] == 128 )#filler record
	{
	}
	elsif ( $f[0] == 129 )#filler record
	{
		read( FILE, $data, 2 ) == 2 or die "Failed to read next block of record!";
		@f = unpack ( 'S', $data );
		my $l = $f[0] - 3;
		seek( FILE, $l, SEEK_CUR );# == $l or die "Error while reading filler record";
	}
	elsif ( $f[0] == 0 )#filler record
	{
		#seek( FILE, 31, SEEK_CUR );# == $l or die "Error while reading filler record";
		read( FILE, $data, 31 );
		my @d = unpack ( 'H[62]', $data );
		print("FILLER0: $d[0]\n");
	}
	elsif ( $f[0] == 1 )#date time change record. 
	#This record is generated when the time in EWSD is changed
	#due to time correction, or daylight savings time correction.
	{
		seek( FILE, 31, SEEK_CUR );
		#read( FILE, $data, 31 );
		#my @d = unpack ( 'H[62]', $data );
		#print("DateTime changed: $d[0]\n");
	}
	elsif ( $f[0] == 2 )#breakdown record
	{
		seek( FILE, 31, SEEK_CUR );
	}
	else
	{
		die "Unknown record identifier: $f[0], at ".tell(FILE)."\n";
	}
}

close( FILE );


if ( !-e $destlog_path )
{
	mkdir $destlog_path;
}

foreach my $key ( keys %DATA )
{
    my ( $YY, $MM, $DD, $HH ) = split( /_/, $key );
	my $path = $destlog_path.$YY;

	if ( !-e $path )
	{
		mkdir $path;
	}
	
	$path = $path."/".$MM;

	if ( !-e $path )
	{
		mkdir $path;
	}

	my $file = $path."/".$DD."_".$HH;

	open( FILE, ">".$file ) or die "Can't create file '$file': $!\n";
	print FILE $DATA{$key};
	close( FILE );

	my $err = `$zip -m -j $path/${DD}_${HH}.zip $file`;
}

foreach my $key ( keys %shortNumbers )
{
	print "$key\n";
}

sub numA2e164
{
	my $num = shift;
	#my $lacLength = shift;print "L:$lacLength N:$num\n";
	chomp($num);
	$num =~ s/^(383\d+)$/7$1/ ;
	return $num;
}

sub numB2e164
{
	my $num = shift;
	chomp($num);

	if ($num =~ s/^810(\d+)$/$1/){;}
	elsif ($num =~ s/^8(\d+)$/7$1/){;}
	elsif ($num =~ /^0(\d+)$/){;}
	elsif ($num =~ s/^(\d{7}\d*)$/7383$1/){;}
	#TODO:
	#elsif ($num =~ /^(7383\d{7})$/){;}
	else { 
#		$shortNumbers{$num} = $num;
#		return undef;
	}

	return $num;
}